/*
FBullCowGame class logic
Logic for the game Bulls and Cows. 
This is the Controller.
This is where the logic is written, for user interaction, see main.cpp.
[C] Copyright 2018: Pranjal Rastogi|Pjr incorporated|Pjr DevSys.
*/

#pragma once
#include "FBullCowGame.h"
#include <map>

// to make unreal friendly
#define TMap std::map
using FString = std::string;
using int32 = int;

FBullCowGame::FBullCowGame() { Reset(); } // constructor

int32 FBullCowGame::GetCurrentTry() const { return MyCurrentTry; }
int32 FBullCowGame::GetHiddenWordLength() const { return MyHiddenWord.length(); }
bool FBullCowGame::IsGameWon() const { return bGameIsWon; }

int32 FBullCowGame::GetMaxTries() const
{
	TMap <int32, int32> WordLengthToMaxTries{ {3,4} , {4,6} , {5,9} , {6,14}, {7,20} };
	return WordLengthToMaxTries[MyHiddenWord.length()]; 
}


EGuessStatus FBullCowGame::CheckGuessValidity(FString Guess) const
{
	if (!IsIsogram(Guess)) 
	{
		return EGuessStatus::Not_Isogram;
	}
	else if (!IsLowercase(Guess)) 
	{
		return EGuessStatus::Not_Lowercase; 
	}
	else if (Guess.length() != GetHiddenWordLength()) 
	{
		return EGuessStatus::Wrong_Length;
	}
	else
	{
		return EGuessStatus::OK;
	}

}

void FBullCowGame::Reset()
{
	const FString HIDDEN_WORD = "dog"; // MUST BE ISOGRAM
	MyHiddenWord = HIDDEN_WORD;

	MyCurrentTry = 1;
	bGameIsWon = false;
	return;
}

// recieves a Valid guess, and increments try and returns count.
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTry++;
	FBullCowCount BullCowCount;

	// loop through all letters
	int32 WordLength = MyHiddenWord.length(); // assuming guess length is equal to hidden word length

	for (int32 MHWChar = 0; MHWChar < WordLength; MHWChar++)
	{
		// compare them under the word
		for (int32 GChar = 0; GChar < WordLength; GChar++)
		{
			if (Guess[GChar] == MyHiddenWord[MHWChar])
			{
				if (MHWChar == GChar)
				{
					BullCowCount.Bulls++;
				}
				else
				{
					BullCowCount.Cows++;
				}
			}
		}
	}
	if (BullCowCount.Bulls == WordLength)
	{
		bGameIsWon = true;
	}
	else
	{
		bGameIsWon = false;
	}

	return BullCowCount;
}

bool FBullCowGame::IsIsogram(FString Word) const
{
	// consider 0 or 1 letters as isograms
	if (Word.length() <= 1) { return true; }
	
	TMap<char, bool> LetterSeen; // setup map
	for (auto Letter : Word) 
	{
		Letter = tolower(Letter); // handle case
	
		if (LetterSeen[Letter]) // If letter is in map 
		{
			return false; // not isogram
		}
		else
		{
			LetterSeen[Letter] = true;
		}
	}
		
	return true; // for cases where \0 is entered {very rare}
}

bool FBullCowGame::IsLowercase(FString Word) const
{
	for (auto Letter : Word)  
	{
		if (!islower(Letter)) // if not lowercase letter
		{
			return false;
		} 	
	}
	return true; 
}
