/* 
Console Application for a game called Bulls and Cows
This is the View Controller and is the View in a MVC pattern.
This is reponsible for user interacton. 
This makes use of the FBullCowGame class, where the logic is written. 
[C] Copyright 2018: Pranjal Rastogi|Pjr incorporated|Pjr DevSys.
*/
/*
NOTES FOR VERSION 2.0 
TODO: ALLOW USER TO CHOOSE WORD LENGTH! and allow game to exit at any time with a key press.
*/

#pragma once
#include <iostream>
#include <string>
#include "FBullCowGame.h"

// to make unreal friendly
using FText = std::string;
using int32 = int;

void PrintGameIntro();
void PlayGame();
FText GetValidGuess();
bool AskToPlayAgain();
void PrintGameSummary();

FBullCowGame BCGame; // instantiate a new game, which we reuse

// App starts here.
int main()
{
	bool bPlayAgain = false;
	do {
		PrintGameIntro();
		PlayGame();
		bPlayAgain = AskToPlayAgain();
	} while (bPlayAgain == true);
	return 0;
}

void PrintGameIntro() 
{
	std::cout << "\n\nWelcome to Bulls and Cows!\n\n";     

	std::cout << R"(          \/        \/                                              \'.____.'/)" << std::endl;
	std::cout << R"(           \'.____.'/"                                             __'-.  .-'__                         .--.)" << std::endl;
	std::cout << R"(          __'-.  .-'__                         ,--,                '_i:'oo':i_'---...____...----i"""-.-'.-"\\)" << std::endl;
	std::cout << R"(           '_i:'oo':i_'---...____...----i"""-.-'.-"\\                /._  _.\       :       /   '._   ;/    ;'-._)" << std::endl;
	std::cout << R"(            /._  _.\                          /     \\-.            (  o  o  )       '-.__.'       '. '.     '-.")" << std::endl;
	std::cout << R"(           \  o  o  /                         |     '-."             '-.__.-' _.--.       COW       '-.:)" << std::endl;
	std::cout << R"(            \--__--/            BULL          :                       : '-'  /     ;   _..--,  /       ;)" << std::endl;
	std::cout << R"(             :                        /       ;                       :      '-._.-'  ;     ; :       :)" << std::endl;
	std::cout << R"(             :                       :       :                         :  `      .'    '-._.' :      /)" << std::endl;
	std::cout << R"(              :  `      .'           :      /                           \  :    /    ____....--\    :)" << std::endl;
	std::cout << R"(               \  :    /    ____....--\    :                            '._\  :"""""    '.     !.   :)" << std::endl;
	std::cout << R"(              '._\  :"""""           !.  \  ;                              : |: :           'www'| \ '|)" << std::endl;
	std::cout << R"(                 : |: :              | \ '| :                              | || |              : |  | :)" << std::endl;
	std::cout << R"(                 | || |              : |  | :                             | || |             .' !  | |)" << std::endl;
	std::cout << R"(                 | || |             .' !  | |                            .' !| |            /__I   | |)" << std::endl;
	std::cout << R"(                .' !| |            /__I   | |                           /__I.' !                  .' !)" << std::endl;
	std::cout << R"(               /__I.' !                  .' !                              /__I                  /__I)" << std::endl;
	std::cout << R"(                  /__I                  /__I)" << std::endl << std::endl;

	std::cout << "Can you guess the " << BCGame.GetHiddenWordLength();
	std::cout << " letter isogram (a word without repeating letters) that is hardcoded inside my code?\n\n";
	std::cout << "Bulls mean correct place and correct letter, while\nCows mean correct letter only.\n";
	std::cout << std::endl;
	return;
}

// plays a single game till completed
void PlayGame()
{
	BCGame.Reset();
	int32 MaxTries = BCGame.GetMaxTries();
	
	// loop asking from guesses while game 
	// is NOT won, and tries are still remaining.
	while (!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries) 
	{
		FText Guess = GetValidGuess();
		// submit valid guess to game and recieve counts
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);

		std::cout << "Bulls = " << BullCowCount.Bulls;
		std::cout << " Cows = " << BullCowCount.Cows << "\n\n";
		
	}
	PrintGameSummary();
	
	return;
}

FText GetValidGuess() 
{
	FText Guess = "";
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	do
	{
		// get a guess
		int32 CurrentTry = BCGame.GetCurrentTry();
		std::cout << "Try " << CurrentTry << " of " << BCGame.GetMaxTries() << ": Enter your guess: ";
		std::getline(std::cin, Guess);

		// check guess validitiy.
		Status = BCGame.CheckGuessValidity(Guess);
		switch (Status)
		{
		case EGuessStatus::Wrong_Length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n\n";
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "Please enter an isogram (A word where there are no repeating letters).\n\n";
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "Please enter the isogram in only lowercase alphabets.\n\n";
			break;
		default: // assume guess is valid.
			break;
		}
	} while (Status != EGuessStatus::OK); //keep looping untl no errors
	return Guess;
}

bool AskToPlayAgain()
{

	std::cout << "Do you want to play again with the same hardcoded isogram [Y/N]? ";
	FText Response = "";
	std::getline(std::cin, Response);
	return ((Response[0] == 'y') || (Response[0] == 'Y'));;
}

void PrintGameSummary()
{
	if (BCGame.IsGameWon())
	{
		std::cout << "Good Job. You just won this EXTREMELY difficult game.\n";
	}
	else
	{
		std::cout << "Better luck next time. You just lost or quit this EXTREMELY difficult game.\n";
	}
	return;
}
